from enum import Enum
from datetime import datetime
import uuid
import json
from coinbase_advanced_trader.cb_auth import CBAuth
from coinbase_advanced_trader.config import API_KEY, API_SECRET

# Initialize the single instance of CBAuth
cb_auth = CBAuth()


class Side(Enum):
    BUY = 1
    SELL = 0


class Method(Enum):
    POST = "POST"
    GET = "GET"


def generate_client_order_id():
    return str(uuid.uuid4())


def set_credentials(api_key, api_secret):
    global cb_auth
    cb_auth = CBAuth(api_key, api_secret)


def listAccounts(limit=49, cursor=None):
    return cb_auth(Method.GET.value, '/api/v3/brokerage/accounts', {'limit': limit, 'cursor': cursor})


def getAccount(account_uuid):
    return cb_auth(Method.GET.value, f'/api/v3/brokerage/accounts/{account_uuid}')


def createOrder(client_order_id, product_id, side, order_type, order_configuration):
    """
    Create an order with the given parameters.

    :param client_order_id: A unique ID generated by the client for this order.
    :param product_id: The ID of the product to order.
    :param side: The side of the order (e.g., 'buy' or 'sell').
    :param order_type: The type of order (e.g., 'limit_limit_gtc').
    :param order_configuration: A dictionary containing order details such as price, size, and post_only.
    :return: A dictionary containing the response from the server.
    """
    payload = {
        "client_order_id": client_order_id,
        "product_id": product_id,
        "side": side,
        "order_configuration": {
            order_type: order_configuration
        }
    }
    # print("Payload being sent to server:", payload)  # For debugging
    return cb_auth(Method.POST.value, '/api/v3/brokerage/orders', payload)


def cancelOrders(order_ids):
    body = json.dumps({"order_ids": order_ids})
    return cb_auth(Method.POST.value, '/api/v3/brokerage/orders/batch_cancel', body)


def listOrders(**kwargs):
    return cb_auth(Method.GET.value, '/api/v3/brokerage/orders/historical/batch', kwargs)


def listFills(**kwargs):
    return cb_auth(Method.GET.value, '/api/v3/brokerage/orders/historical/fills', kwargs)


def getOrder(order_id):
    return cb_auth(Method.GET.value, f'/api/v3/brokerage/orders/historical/{order_id}')


def listProducts(**kwargs):
    return cb_auth(Method.GET.value, '/api/v3/brokerage/products', kwargs)


def getProduct(product_id):
    return cb_auth(Method.GET.value, f'/api/v3/brokerage/products/{product_id}')


def getProductCandles(product_id, start, end, granularity):
    params = {
        'start': start,
        'end': end,
        'granularity': granularity
    }
    return cb_auth(Method.GET.value, f'/api/v3/brokerage/products/{product_id}/candles', params)


def getMarketTrades(product_id, limit):
    return cb_auth(Method.GET.value, f'/api/v3/brokerage/products/{product_id}/ticker', {'limit': limit})


def getTransactionsSummary(start_date, end_date, user_native_currency='USD', product_type='SPOT'):
    params = {
        'start_date': start_date.strftime('%Y-%m-%dT%H:%M:%SZ'),
        'end_date': end_date.strftime('%Y-%m-%dT%H:%M:%SZ'),
        'user_native_currency': user_native_currency,
        'product_type': product_type
    }
    return cb_auth(Method.GET.value, '/api/v3/brokerage/transaction_summary', params)
